@@ -36,10 +36,10 @@
 #include <QtCore/QRegularExpression>
 #include <QtCore/QStringBuilder>
 
-#include "lib/events/roommessageevent.h"
-#include "lib/user.h"
-#include "lib/connection.h"
-#include "lib/settings.h"
+#include <events/roommessageevent.h>
+#include <user.h>
+#include <connection.h>
+#include <settings.h>
 #include "models/messageeventmodel.h"
 #include "imageprovider.h"
 #include "chatedit.h"
@@ -60,6 +60,8 @@ ChatRoomWidget::ChatRoomWidget(QWidget* parent)
         qmlRegisterUncreatableType<User>("QMatrixClient", 1, 0, "User",
             "User objects can only be created by libqmatrixclient");
         qmlRegisterType<Settings>("QMatrixClient", 1, 0, "Settings");
+        qmlRegisterUncreatableType<RoomMessageEvent>("QMatrixClient", 1, 0,
+            "RoomMessageEvent", "RoomMessageEvent is uncreatable");
     }
 
     m_roomAvatar = new QLabel();
@@ -77,15 +79,15 @@ ChatRoomWidget::ChatRoomWidget(QWidget* parent)
     topicSeparator->setFrameShape(QFrame::HLine);
 
     m_timelineWidget = new timelineWidget_t;
+    qDebug() << "Rendering QML with"
+             << timelineWidget_t::staticMetaObject.className();
+    auto* qmlContainer =
 #ifdef DISABLE_QQUICKWIDGET
-    qDebug() << "Using QQuickView wrapper instead of QQuickWidget";
-    auto* container = QWidget::createWindowContainer(m_timelineWidget, this);
-    container->
+            QWidget::createWindowContainer(m_timelineWidget, this);
 #else
-    qDebug() << "Using QQuickWidget to render QML";
-    m_timelineWidget->
+            m_timelineWidget;
 #endif // Use different objects but the same method with the same parameters
-            setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+    qmlContainer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
 
     m_timelineWidget->setResizeMode(timelineWidget_t::SizeRootObjectToView);
 
@@ -124,13 +126,7 @@ ChatRoomWidget::ChatRoomWidget(QWidget* parent)
     layout->addLayout(headerLayout);
 
     layout->addWidget(topicSeparator);
-    layout->addWidget(
-#ifdef DISABLE_QQUICKWIDGET
-                container
-#else
-                m_timelineWidget
-#endif
-    );
+    layout->addWidget(qmlContainer);
     layout->addWidget(m_currentlyTyping);
     layout->addWidget(m_chatEdit);
     setLayout(layout);
@@ -151,6 +147,7 @@ void ChatRoomWidget::setRoom(QuaternionRoom* room)
 
     if( m_currentRoom )
     {
+        m_currentRoom->setDisplayed(false);
         m_currentRoom->setCachedInput( m_chatEdit->toPlainText() );
         roomHistories.insert(m_currentRoom, m_chatEdit->history());
         m_currentRoom->connection()->disconnect(this);
@@ -196,6 +193,7 @@ void ChatRoomWidget::setRoom(QuaternionRoom* room)
             qWarning() << "Logged out, escaping the room";
             setRoom(nullptr);
         });
+        m_currentRoom->setDisplayed(true);
     } else
         m_imageProvider->setConnection(nullptr);
     updateHeader();
@@ -228,7 +226,7 @@ void ChatRoomWidget::updateHeader()
         auto topic = m_currentRoom->topic();
         auto prettyTopic = topic.isEmpty() ?
                 tr("(no topic)") : m_currentRoom->prettyPrint(topic);
-        m_topicLabel->setText("<strong>" % m_currentRoom->name() %
+        m_topicLabel->setText("<strong>" % m_currentRoom->displayName() %
                               "</strong><br />" % prettyTopic);
         auto avatarSize = m_topicLabel->heightForWidth(width());
         m_roomAvatar->setPixmap(
@@ -252,9 +250,9 @@ void ChatRoomWidget::encryptionChanged()
             : DefaultPlaceholderText);
 }
 
-void ChatRoomWidget::insertMention(QString author)
+void ChatRoomWidget::insertMention(QMatrixClient::User* user)
 {
-    m_chatEdit->insertMention(std::move(author));
+    m_chatEdit->insertMention(user->displayname(m_currentRoom));
 }
 
 void ChatRoomWidget::focusInput()
@@ -294,13 +292,13 @@ QString ChatRoomWidget::doSendInput()
 
     if (!text.startsWith('/'))
     {
-        m_currentRoom->postMessage(text);
+        m_currentRoom->postPlainText(text);
         return {};
     }
     if (text[1] == '/')
     {
         text.remove(0, 1);
-        m_currentRoom->postMessage(text);
+        m_currentRoom->postPlainText(text);
         return {};
     }
 
@@ -311,7 +309,8 @@ QString ChatRoomWidget::doSendInput()
     static const QRegularExpression
             CommandRe { "^/([^ ]+)( +(.*))?\\s*$", ReFlags },
             RoomIdRE { "^[#!][-0-9a-z._=]+:\\S+$", ReFlags },
-            UserIdRE { "^@[-0-9a-z._=]+:\\S+$", ReFlags };
+            UserIdRE { "^@[-0-9a-z._=]+:\\S+$", ReFlags },
+            HtmlTagRE { "<[^>]+>", ReFlags };
 
     // Process a command
     const auto matches = CommandRe.match(text);
@@ -337,7 +336,7 @@ QString ChatRoomWidget::doSendInput()
                   " Start with // to send this line literally");
 
     // Commands available only in the room context
-    using QMatrixClient::Room;
+    using namespace QMatrixClient;
     if (command == "leave" || command == "part")
     {
         if (!argString.isEmpty())
@@ -374,34 +373,50 @@ QString ChatRoomWidget::doSendInput()
     {
         const auto args = lazySplitRef(argString, ' ', 2);
         if (args.front().isEmpty())
-            return tr("/%1 <memberId> <reason>").arg(command.toString());
+            return tr("/%1 <userId> <reason>").arg(command.toString());
         if (!UserIdRE.match(args.front()).hasMatch())
             return tr("%1 doesn't look like a user id")
                     .arg(args.front());
 
-        auto* user = m_currentRoom->user(args.front());
-        if (m_currentRoom->memberJoinState(user)
-                != QMatrixClient::JoinState::Join)
-            return tr("%1 is not a member of this room")
-                    .arg(user->fullName(m_currentRoom));
-
         if (command == "ban")
-            m_currentRoom->ban(user->id(), args.back());
-        else
+            m_currentRoom->ban(args.front(), args.back());
+        else {
+            auto* user = m_currentRoom->user(args.front());
+            if (m_currentRoom->memberJoinState(user) != JoinState::Join)
+                return tr("%1 is not a member of this room")
+                        .arg(user->fullName(m_currentRoom));
             m_currentRoom->kickMember(user->id(), args.back());
+        }
         return {};
     }
     if (command == "unban")
     {
         if (argString.isEmpty())
-            return tr("/unban <memberId>");
+            return tr("/unban <userId>");
         if (!argString.contains(UserIdRE))
             return tr("/unban argument doesn't look like a user ID");
 
         m_currentRoom->unban(argString);
         return {};
     }
-    using MsgType = QMatrixClient::RoomMessageEvent::MsgType;
+    if (command == "ignore" || command == "unignore")
+    {
+        if (argString.isEmpty())
+            return tr("/ignore <userId>");
+        if (!argString.contains(UserIdRE))
+            return tr("/ignore argument doesn't look like a user ID");
+
+        if (auto* user = m_currentRoom->user(argString))
+        {
+            if (command == "ignore")
+                user->ignore();
+            else
+                user->unmarkIgnore();
+            return {};
+        }
+        return tr("Couldn't find user %1 on the server").arg(argString);
+    }
+    using MsgType = RoomMessageEvent::MsgType;
     if (command == "me")
     {
         if (argString.isEmpty())
@@ -418,7 +433,7 @@ QString ChatRoomWidget::doSendInput()
     }
     if (command == "shrug") // Peeked at Discord
     {
-        m_currentRoom->postMessage("¯\\_(ツ)_/¯");
+        m_currentRoom->postPlainText("¯\\_(ツ)_/¯");
         return {};
     }
     if (command == "topic")
@@ -445,24 +460,30 @@ QString ChatRoomWidget::doSendInput()
         {
             if (auto* room = m_currentRoom->connection()->room(args.front()))
             {
-                room->postMessage(args.back());
+                room->postPlainText(args.back());
                 return {};
             }
-            else
-                return tr("%1 doesn't seem to have joined room %2")
-                        .arg(m_currentRoom->localUser()->id(),
-                             args.front());
+            return tr("%1 doesn't seem to have joined room %2")
+                    .arg(m_currentRoom->localUser()->id(), args.front());
         }
         if (UserIdRE.match(args.front()).hasMatch())
         {
             m_currentRoom->connection()->doInDirectChat(args.front(),
-                [msg=args.back()] (Room* dc) { dc->postMessage(msg); });
+                [msg=args.back()] (Room* dc) { dc->postPlainText(msg); });
             return {};
         }
 
         return tr("%1 doesn't look like a user id or room alias")
                 .arg(args.front());
     }
+    if (command == "html")
+    {
+        // Very crude HTML-to-plaintext conversion - just strip all the tags
+        auto plainText = argString;
+        plainText.replace(HtmlTagRE, QString());
+        m_currentRoom->postHtmlText(plainText, argString);
+        return {};
+    }
     if (command == "query" || command == "dc")
     {
         if (argString.isEmpty())
@@ -525,9 +546,9 @@ void ChatRoomWidget::saveFileAs(QString eventId)
         m_currentRoom->downloadFile(eventId, QUrl::fromLocalFile(fileName));
 }
 
-void ChatRoomWidget::onMessageShownChanged(QString eventId, bool shown)
+void ChatRoomWidget::onMessageShownChanged(const QString& eventId, bool shown)
 {
-    if (!m_currentRoom)
+    if (!m_currentRoom || !m_currentRoom->displayed())
         return;
 
     // A message can be auto-marked as read (as soon as the user is active), if:
@@ -557,8 +578,8 @@ void ChatRoomWidget::onMessageShownChanged(QString eventId, bool shown)
     const auto iter = m_currentRoom->findInTimeline(eventId);
     if (iter == m_currentRoom->timelineEdge())
     {
-        qCritical() <<  "Event" << eventId << "is not in the timeline";
-        Q_ASSERT(false);
+        qWarning() << "Event" << eventId
+                   << "is not in the timeline (local echo?)";
         return;
     }
     const auto timelineIndex = iter->index();
