@@ -62,11 +62,11 @@ class ChatRoomWidget: public QWidget
         void setRoom(QuaternionRoom* room);
         void updateHeader();
 
-        void insertMention(QString author);
+        void insertMention(QMatrixClient::User* user);
         void focusInput();
 
         void typingChanged();
-        void onMessageShownChanged(QString eventId, bool shown);
+        void onMessageShownChanged(const QString& eventId, bool shown);
         void markShownAsRead();
         void saveFileAs(QString eventId);
 
