@@ -18,16 +18,16 @@
  **************************************************************************/
 
 #include <QtWidgets/QApplication>
-#include <QtWidgets/QSplashScreen>
 #include <QtCore/QTranslator>
 #include <QtCore/QLibraryInfo>
 #include <QtCore/QCommandLineParser>
 #include <QtCore/QDebug>
+#include <QtCore/QStandardPaths>
 
 #include "networksettings.h"
 #include "mainwindow.h"
 #include "activitydetector.h"
-#include "lib/settings.h"
+#include <settings.h>
 
 int main( int argc, char* argv[] )
 {
@@ -35,7 +35,7 @@ int main( int argc, char* argv[] )
     QApplication::setOrganizationName("QMatrixClient");
     QApplication::setApplicationName("quaternion");
     QApplication::setApplicationDisplayName("Quaternion");
-    QApplication::setApplicationVersion("0.0.9.2");
+    QApplication::setApplicationVersion("0.0.9.3");
 
     QMatrixClient::Settings::setLegacyNames("Quaternion", "quaternion");
 
@@ -47,47 +47,59 @@ int main( int argc, char* argv[] )
         QApplication::postEvent(qApp, new QEvent(QEvent::Quit));
     });
 
+    QCommandLineParser parser;
+    parser.setApplicationDescription(QApplication::translate("main",
+            "Quaternion - an IM client for the Matrix protocol"));
+    parser.addHelpOption();
+    parser.addVersionOption();
+
+    QList<QCommandLineOption> options;
+    QCommandLineOption locale { "locale",
+        QApplication::translate("main", "Override locale"),
+        QApplication::translate("main", "locale") };
+    options.append(locale);
+    QCommandLineOption debug { "debug",
+        QApplication::translate("main", "Display debug information") };
+    debug.setHidden(true); // FIXME, #415; also, setHidden is obsolete in Qt 5.11
+    options.append(debug);
+    // Add more command line options before this line
+
+    if (!parser.addOptions(options))
+        Q_ASSERT_X(false, __FUNCTION__,
+                   "Command line options are improperly defined, fix the code");
+    parser.process(app);
+
+    const auto overrideLocale = parser.value(locale);
+    if (!overrideLocale.isEmpty())
+    {
+        QLocale::setDefault(overrideLocale);
+        qInfo() << "Using locale" << QLocale().name();
+    }
+
     QTranslator qtTranslator;
     qtTranslator.load(QLocale(), "qt", "_",
                       QLibraryInfo::location(QLibraryInfo::TranslationsPath));
     app.installTranslator(&qtTranslator);
 
     QTranslator appTranslator;
-    appTranslator.load(QLocale(), "quaternion", ".", "i18n");
+    if (!appTranslator.load(QLocale(), "quaternion", "_"))
+        appTranslator.load(QLocale(), "quaternion", "_",
+            QStandardPaths::locate(QStandardPaths::AppLocalDataLocation,
+            "translations", QStandardPaths::LocateDirectory));
     app.installTranslator(&appTranslator);
 
-    QCommandLineParser parser;
-    parser.setApplicationDescription(QApplication::translate("main", "An IM client for the Matrix protocol"));
-    parser.addHelpOption();
-    parser.addVersionOption();
-
-    QCommandLineOption debug("debug", QApplication::translate("main", "Display debug information"));
-    parser.addOption(debug);
-
-    parser.process(app);
-    bool debugEnabled = parser.isSet(debug);
-    qDebug() << "Debug: " << debugEnabled;
-
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 10, 0))
     app.setAttribute(Qt::AA_DisableWindowContextHelpButton);
 #endif
 
-//    bool splashEnabled = !QSettings().value("UI/suppress_splash").toBool();
-//    QSplashScreen splash(QPixmap(":/icon.png"));
-//    if (splashEnabled)
-//    {
-//        splash.show();
-//        app.processEvents();
-//    }
-
     QMatrixClient::NetworkSettings().setupApplicationProxy();
 
     MainWindow window;
-    if( debugEnabled )
+    if (parser.isSet(debug))
+    {
+        qInfo() << "Debug mode enabled";
         window.enableDebug();
-
-//    if (splashEnabled)
-//        splash.finish(&window); // calls app.processEvents()
+    }
 
     ActivityDetector ad(app, window); Q_UNUSED(ad);
     qDebug() << "--- Show time!";
