@@ -27,11 +27,12 @@
 #include "roomdialogs.h"
 #include "systemtrayicon.h"
 
-#include "lib/csapi/joining.h"
-#include "lib/connection.h"
-#include "lib/networkaccessmanager.h"
-#include "lib/settings.h"
-#include "lib/logging.h"
+#include <csapi/joining.h>
+#include <connection.h>
+#include <networkaccessmanager.h>
+#include <settings.h>
+#include <logging.h>
+#include <user.h>
 
 #include <QtCore/QTimer>
 #include <QtCore/QDebug>
@@ -52,6 +53,7 @@
 #include <QtWidgets/QDialogButtonBox>
 #include <QtWidgets/QFormLayout>
 #include <QtGui/QMovie>
+#include <QtGui/QPixmap>
 #include <QtGui/QCloseEvent>
 #include <QtGui/QDesktopServices>
 
@@ -83,6 +85,8 @@ MainWindow::MainWindow()
     connect( roomListDock, &RoomListDock::roomSelected,
              this, &MainWindow::selectRoom);
     connect( chatRoomWidget, &ChatRoomWidget::showStatusMessage, statusBar(), &QStatusBar::showMessage );
+    connect( userListDock, &UserListDock::userMentionRequested,
+             chatRoomWidget, &ChatRoomWidget::insertMention);
 
     createMenu();
     systemTrayIcon = new SystemTrayIcon(this);
@@ -119,31 +123,110 @@ void summon(QPointer<DialogT>& dlg, DialogArgTs&&... dialogArgs)
     dlg->reactivate();
 }
 
+QAction* MainWindow::addTimelineOptionCheckbox(QMenu* parent,
+    const QString& text, const QString& statusTip, const QString& settingsKey,
+    bool defaultValue)
+{
+    using QMatrixClient::SettingsGroup;
+    auto action =
+        parent->addAction(text,
+            [this,settingsKey] (bool checked)
+            {
+                SettingsGroup("UI").setValue(settingsKey, checked);
+                chatRoomWidget->setRoom(nullptr);
+                chatRoomWidget->setRoom(currentRoom);
+            });
+    action->setStatusTip(statusTip);
+    action->setCheckable(true);
+    action->setChecked(
+        SettingsGroup("UI").value(settingsKey, defaultValue).toBool());
+    return action;
+}
+
 void MainWindow::createMenu()
 {
+    using QMatrixClient::Settings;
+
     // Connection menu
     connectionMenu = menuBar()->addMenu(tr("&Accounts"));
 
-    connectionMenu->addAction(tr("&Login..."), this, [=]{ showLoginWindow(); } );
+    connectionMenu->addAction(QIcon::fromTheme("im-user"), tr("&Login..."),
+        this, [=]{ showLoginWindow(); } );
 
     connectionMenu->addSeparator();
     // Account submenus will be added in this place - see addConnection()
     accountListGrowthPoint = connectionMenu->addSeparator();
 
-    connectionMenu->addAction(tr("&Quit"),
-        qApp, &QApplication::closeAllWindows, QKeySequence::Quit);
+    connectionMenu->addAction(QIcon::fromTheme("application-exit"),
+        tr("&Quit"), qApp, &QApplication::closeAllWindows, QKeySequence::Quit);
 
     // View menu
     auto viewMenu = menuBar()->addMenu(tr("&View"));
 
-    viewMenu->addAction(roomListDock->toggleViewAction());
-    viewMenu->addAction(userListDock->toggleViewAction());
+    auto dockPanesMenu = viewMenu->addMenu(
+        QIcon::fromTheme("labplot-editvlayout"), tr("Dock &panels"));
+    roomListDock->toggleViewAction()
+            ->setStatusTip("Show/hide Rooms dock panel");
+    dockPanesMenu->addAction(roomListDock->toggleViewAction());
+    userListDock->toggleViewAction()
+        ->setStatusTip("Show/hide Users dock panel");
+    dockPanesMenu->addAction(userListDock->toggleViewAction());
+
+    viewMenu->addSeparator();
+
+    auto showEventsMenu = viewMenu->addMenu(tr("&Display in timeline"));
+    addTimelineOptionCheckbox(
+        showEventsMenu,
+        tr("Normal &join/leave events"),
+        tr("Show join and leave events that don't couple in a no-change pair"),
+        QStringLiteral("show_joinleave"),
+        true
+    );
+    addTimelineOptionCheckbox(
+        showEventsMenu,
+        tr("&Redacted events"),
+        tr("Show redacted events in the timeline as 'Redacted'"
+           " instead of hiding them entirely"),
+        QStringLiteral("show_redacted")
+    );
+    addTimelineOptionCheckbox(
+        showEventsMenu,
+        tr("&No-effect activity"),
+        tr("Show join and leave events that couple in a no-change pair,"
+           " possibly with redactions between"),
+        QStringLiteral("show_spammy")
+    );
+
+    viewMenu->addSeparator();
+
+    viewMenu->addAction(tr("Edit tags order"), [this]
+    {
+        static const auto SettingsKey = QStringLiteral("tags_order");
+        QMatrixClient::SettingsGroup sg { QStringLiteral("UI/RoomsDock") };
+        const auto savedOrder = sg.get<QStringList>(SettingsKey).join('\n');
+        bool ok;
+        const auto newOrder = QInputDialog::getMultiLineText(this,
+                tr("Edit tags order"),
+                tr("Tags can be wildcarded by * next to dot(s)\n"
+                   "Clear the box to reset to defaults\n"
+                   "org.qmatrixclient. tags: invite, left, direct, none"),
+                savedOrder, &ok);
+        if (ok)
+        {
+            if (newOrder.isEmpty())
+                sg.remove(SettingsKey);
+            else if (newOrder != savedOrder)
+                sg.setValue(SettingsKey, newOrder.split('\n'));
+            roomListDock->updateSortingMode();
+        }
+    });
 
     // Room menu
     auto roomMenu = menuBar()->addMenu(tr("&Room"));
 
     roomSettingsAction =
-        roomMenu->addAction(tr("Change room &settings..."), [this]
+        roomMenu->addAction(QIcon::fromTheme("user-group-properties"),
+        tr("Change room &settings..."), [this]
         {
             static QHash<QuaternionRoom*, QPointer<RoomSettingsDialog>> dlgs;
             summon(dlgs[currentRoom], currentRoom, this);
@@ -151,20 +234,29 @@ void MainWindow::createMenu()
     roomSettingsAction->setDisabled(true);
     roomMenu->addSeparator();
     createRoomAction =
-        roomMenu->addAction(tr("Create &new room..."), [this]
+        roomMenu->addAction(QIcon::fromTheme("user-group-new"),
+        tr("Create &new room..."), [this]
         {
             static QPointer<CreateRoomDialog> dlg;
             summon(dlg, connections, this);
         });
     createRoomAction->setDisabled(true);
-    roomMenu->addAction(tr("&Join room..."), [=]{ joinRoom(); } );
+    roomMenu->addAction(QIcon::fromTheme("list-add-user"),
+        tr("&Direct chat..."), [=]{ directChat(); });
+    roomMenu->addAction(QIcon::fromTheme("list-add"), tr("&Join room..."),
+        [=]{ joinRoom(); } );
     roomMenu->addSeparator();
-    roomMenu->addAction(tr("&Close current room"),
-        [this] { selectRoom(nullptr); }, QKeySequence::Close);
+    roomMenu->addAction(QIcon::fromTheme("window-close"),
+        tr("&Close current room"), [this] { selectRoom(nullptr); },
+        QKeySequence::Close);
 
     // Settings menu
     auto settingsMenu = menuBar()->addMenu(tr("&Settings"));
-    using QMatrixClient::Settings;
+
+    // Help menu
+    auto helpMenu = menuBar()->addMenu(tr("&Help"));
+    helpMenu->addAction(QIcon::fromTheme("help-about"), tr("&About"),
+        [=]{ showAboutWindow(); });
 
     {
         auto notifGroup = new QActionGroup(this);
@@ -188,7 +280,9 @@ void MainWindow::createMenu()
         fullNotif->setStatusTip(
             tr("Show notifications and activate the window"));
 
-        auto notifMenu = settingsMenu->addMenu(tr("Notifications"));
+        auto notifMenu = settingsMenu->addMenu(
+            QIcon::fromTheme("preferences-desktop-notification"),
+            tr("Notifications"));
         for (auto a: {noNotif, gentleNotif, fullNotif})
         {
             a->setCheckable(true);
@@ -223,7 +317,8 @@ void MainWindow::createMenu()
         xchatLayout->setStatusTip(
             tr("The layout with author labels to the left from each message"));
 
-        auto layoutMenu = settingsMenu->addMenu(tr("Timeline layout"));
+        auto layoutMenu = settingsMenu->addMenu(QIcon::fromTheme("table"),
+            tr("Timeline layout"));
         for (auto a: {defaultLayout, xchatLayout})
         {
             a->setCheckable(true);
@@ -237,22 +332,16 @@ void MainWindow::createMenu()
         else
             defaultLayout->setChecked(true);
     }
-    auto autoloadImages =
-        settingsMenu->addAction(tr("Load full-size images at once"),
-            [this] (bool checked)
-            {
-                Settings().setValue("UI/autoload_images", checked);
-                chatRoomWidget->setRoom(nullptr);
-                chatRoomWidget->setRoom(currentRoom);
-            });
-    autoloadImages->setStatusTip(
-        tr("Automatically download a full-size image instead of a thumbnail"));
-    autoloadImages->setCheckable(true);
-    autoloadImages->setChecked(
-        Settings().value("UI/autoload_images", true).toBool());
+    addTimelineOptionCheckbox(
+        settingsMenu,
+        tr("Load full-size images at once"),
+        tr("Automatically download a full-size image instead of a thumbnail"),
+        QStringLiteral("autoload_images"), true
+    );
 
     settingsMenu->addSeparator();
-    settingsMenu->addAction(tr("Configure &network proxy..."), [this]
+    settingsMenu->addAction(QIcon::fromTheme("preferences-system-network"),
+        tr("Configure &network proxy..."), [this]
     {
         static QPointer<NetworkConfigDialog> dlg;
         summon(dlg, this);
@@ -417,7 +506,7 @@ void MainWindow::addConnection(Connection* c, const QString& deviceName)
             msgBox.setTextFormat(Qt::RichText);
             msgBox.setDetailedText(
                 "Request URL: " + job->requestUrl().toDisplayString() +
-                "\nResponse:\n" + job->rawData());
+                "\nResponse:\n" + job->rawData(65535));
             QPushButton* openUrlButton = nullptr;
             if (job->errorUrl().isEmpty())
                 msgBox.setDefaultButton(QMessageBox::Close);
@@ -440,7 +529,11 @@ void MainWindow::addConnection(Connection* c, const QString& deviceName)
             if (currentRoom == prev)
                 selectRoom(r);
         });
-    connect( c, &Connection::directChatAvailable, this, &MainWindow::selectRoom);
+    connect( c, &Connection::directChatAvailable, this,
+             [this] (Room* r) {
+                 selectRoom(r);
+                 statusBar()->showMessage("Direct chat opened", 2000);
+             });
     connect( c, &Connection::aboutToDeleteRoom, this,
              [this] (Room* r) {
                  if (currentRoom == r)
@@ -454,7 +547,8 @@ void MainWindow::addConnection(Connection* c, const QString& deviceName)
     if (connections.size() < 10)
         menuCaption.prepend('&' % QString::number(connections.size()) % ' ');
     auto accountMenu = new QMenu(menuCaption, connectionMenu);
-    accountMenu->addAction(tr("Show &access token"), this, [=]
+    accountMenu->addAction(QIcon::fromTheme("view-certificate"),
+        tr("Show &access token"), this, [=]
     {
         const QString aToken = c->accessToken();
         auto accountTokenBox = new QMessageBox(QMessageBox::Information,
@@ -470,7 +564,8 @@ void MainWindow::addConnection(Connection* c, const QString& deviceName)
         accountTokenBox->setAttribute(Qt::WA_DeleteOnClose);
         accountTokenBox->show();
     });
-    accountMenu->addAction(tr("&Logout"), this, [=] { logout(c); });
+    accountMenu->addAction(QIcon::fromTheme("system-log-out"), tr("&Logout"),
+                           this, [=] { logout(c); });
     auto menuAction =
             connectionMenu->insertMenu(accountListGrowthPoint, accountMenu);
     connect( c, &Connection::destroyed, connectionMenu, [this, menuAction]
@@ -530,6 +625,87 @@ void MainWindow::showLoginWindow(const QString& statusMessage)
     }
 }
 
+void MainWindow::showAboutWindow()
+{
+    Dialog aboutDialog(tr("About Quaternion"), Dialog::NoExtraButtons,
+                       this, Dialog::NoStatusLine);
+    auto* tabWidget = new QTabWidget();
+    {
+        auto *aboutPage = new QWidget();
+        tabWidget->addTab(aboutPage, tr("&About"));
+
+        auto* layout = new QVBoxLayout(aboutPage);
+
+        auto* imageLabel = new QLabel();
+        imageLabel->setPixmap(QPixmap(":/icon.png"));
+        imageLabel->setAlignment(Qt::AlignHCenter);
+        layout->addWidget(imageLabel);
+
+        auto* labelString =
+                new QLabel("<h1>" + QApplication::applicationDisplayName() + " v" +
+                           QApplication::applicationVersion() + "</h1>");
+        labelString->setAlignment(Qt::AlignHCenter);
+        layout->addWidget(labelString);
+
+        auto* linkLabel =
+                new QLabel("<a href=\"https://matrix.org/docs/projects/client/quaternion.html\">"
+                           + tr("Web page") + "</a>");
+        linkLabel->setAlignment(Qt::AlignHCenter);
+        linkLabel->setOpenExternalLinks(true);
+        layout->addWidget(linkLabel);
+
+        layout->addWidget(
+                    new QLabel(tr("Copyright (C) 2018 QMatrixClient project.")));
+
+#ifdef GIT_SHA1
+        auto* commitLabel = new QLabel(tr("Built from Git, commit SHA:\n") +
+                                          QStringLiteral(GIT_SHA1));
+        commitLabel->setTextInteractionFlags(Qt::TextSelectableByKeyboard|
+                                             Qt::TextSelectableByMouse);
+        layout->addWidget(commitLabel);
+#endif
+
+#ifdef LIB_GIT_SHA1
+        auto* libCommitLabel = new QLabel(new QLabel(tr("Library commit SHA:\n") +
+                                          QStringLiteral(LIB_GIT_SHA1)));
+        libCommitLabel->setTextInteractionFlags(Qt::TextSelectableByKeyboard|
+                                                Qt::TextSelectableByMouse);
+        layout->addWidget(libCommitLabel);
+#endif
+    }
+    {
+        auto* thanksLabel = new QLabel(
+            tr("Original project author: %1")
+            .arg("<a href='https://github.com/Fxrh'>Felix Rohrbach</a>") + "<br/>" +
+            tr("Project leader: %1")
+            .arg("<a href='https://github.com/KitsuneRal'>Kitsune Ral</a>") +
+            "<br/><br/>" +
+            tr("Contributors:") + "<br/>" +
+            "<a href='https://github.com/QMatrixClient/Quaternion/graphs/contributors'>" +
+                tr("Quaternion contributors @ GitHub") + "</a><br/>" +
+            "<a href='https://github.com/QMatrixClient/libqmatrixclient/graphs/contributors'>" +
+                tr("libQMatrixClient contributors @ GitHub") + "</a><br/>" +
+            "<a href='https://lokalise.co/contributors/730769035bbc328c31e863.62506391/'>" +
+                tr("Quaternion translators @ Lokalise.co") + "</a><br/>" +
+            "<br/>" +
+            tr("Made with:") + "<br/>" +
+            "<a href='https://www.qt.io/'>Qt 5</a><br/>"
+            "<a href='https://www.kdevelop.org/'>KDevelop</a><br/>"
+            "<a href='https://www.qt.io/qt-features-libraries-apis-tools-and-ide/#ide'>Qt Creator</a><br/>"
+            "<a href='https://www.jetbrains.com/clion/'>CLion</a><br/>"
+            "<a href='https://lokalise.co'>Lokalise.co</a>"
+        );
+        thanksLabel->setTextInteractionFlags(Qt::TextSelectableByKeyboard|
+                                             Qt::TextBrowserInteraction);
+        thanksLabel->setOpenExternalLinks(true);
+
+        tabWidget->addTab(thanksLabel, tr("&Thanks"));
+    }
+
+    aboutDialog.layout()->addWidget(tabWidget);
+    aboutDialog.exec();
+}
+
 void MainWindow::invokeLogin()
 {
     using namespace QMatrixClient;
@@ -700,6 +876,72 @@ void MainWindow::joinRoom(const QString& roomAlias, Connection* connection)
     });
 }
 
+void MainWindow::directChat(const QString& userId, Connection* connection) {
+    if (connections.isEmpty())
+    {
+        QMessageBox::warning(this, tr("No connections"),
+            tr("Please connect to a server before joining a room"),
+            QMessageBox::Close, QMessageBox::Close);
+        return;
+    }
+
+    if (!connection)
+    {
+        if (currentRoom && !userId.isEmpty())
+        {
+            connection = currentRoom->connection();
+            if (connections.size() > 1)
+            {
+                // Double check the user intention
+                QMessageBox confirmBox(QMessageBox::Question,
+                    tr("Starting direct chat with %1 as %2").arg(userId, connection->userId()),
+                    tr("Start direct chat with %1 under account %2?")
+                        .arg(userId, connection->userId()),
+                    QMessageBox::Ok|QMessageBox::Cancel, this);
+                confirmBox.setButtonText(QMessageBox::Ok, tr("Start Chat"));
+                auto* chooseAccountButton =
+                    confirmBox.addButton(tr("Choose account..."),
+                                         QMessageBox::ActionRole);
+
+                if (confirmBox.exec() == QMessageBox::Cancel)
+                    return;
+                if (confirmBox.clickedButton() == chooseAccountButton)
+                    connection = chooseConnection();
+            }
+        } else
+            connection = connections.size() == 1 ? connections.front() :
+                         chooseConnection();
+    }
+    if (!connection)
+        return; // No default connection and the user discarded the dialog
+
+    QString userName = userId;
+    while (userName.isEmpty())
+    {
+        QInputDialog userInput (this);
+        userInput.setWindowTitle(tr("Enter user id to start direct chat."));
+        userInput.setLabelText(
+            tr("Enter the user id of who you would like to chat with. You will join as %1")
+                .arg(connection->userId()));
+        userInput.setOkButtonText(tr("Start Chat"));
+        userInput.setSizeGripEnabled(true);
+        // TODO: Provide a button to select the joining account
+        if (userInput.exec() == QDialog::Rejected)
+            return;
+        // TODO: Check validity, not only non-emptyness
+        if (!userInput.textValue().isEmpty())
+            userName = userInput.textValue();
+        else
+            QMessageBox::warning(this, tr("No user id specified"),
+                                 tr("Please specify non-empty user id"),
+                                 QMessageBox::Close, QMessageBox::Close);
+    }
+
+    connection->requestDirectChat(userName);
+    statusBar()->showMessage(tr("Starting chat with %1 as %2")
+                                .arg(userName, connection->userId()), 3000);
+}
+
 void MainWindow::getNewEvents(Connection* c)
 {
     Q_ASSERT_X(c, __FUNCTION__, "Attempt to sync on null connection");
