@@ -70,6 +70,8 @@ class MainWindow: public QMainWindow
         void invokeLogin();
         void joinRoom(const QString& roomAlias = {},
                       Connection* connection = nullptr);
+        void directChat(const QString& roomAlias = {},
+                      Connection* connection = nullptr);
         void getNewEvents(Connection* c);
         void gotEvents(Connection* c);
 
@@ -80,6 +82,7 @@ class MainWindow: public QMainWindow
                                          QAuthenticator* auth);
 
         void showLoginWindow(const QString& statusMessage = {});
+        void showAboutWindow();
         void logout(Connection* c);
 
     private:
@@ -105,6 +108,9 @@ class MainWindow: public QMainWindow
         QuaternionRoom* currentRoom = nullptr;
 
         void createMenu();
+        QAction* addTimelineOptionCheckbox(QMenu* parent,
+            const QString& text, const QString& statusTip,
+            const QString& settingsKey, bool defaultValue = false);
         void showFirstSyncIndicator();
         void loadSettings();
         void saveSettings() const;
