@@ -20,9 +20,9 @@
 #pragma once
 
 #include <QtWidgets/QDockWidget>
-#include <QtWidgets/QListView>
+#include <QtWidgets/QTreeView>
 #include <QtCore/QStringListModel>
-#include <QSortFilterProxyModel>
+//#include <QtCore/QSortFilterProxyModel>
 
 class RoomListModel;
 class QuaternionRoom;
@@ -39,30 +39,33 @@ class RoomListDock : public QDockWidget
 
         void addConnection(QMatrixClient::Connection* connection);
 
+    public slots:
+        void updateSortingMode();
+
     signals:
         void roomSelected(QuaternionRoom* room);
 
     private slots:
         void rowSelected(const QModelIndex& index);
         void showContextMenu(const QPoint& pos);
-        void menuMarkReadSelected();
         void addTagsSelected();
-        void menuJoinSelected();
-        void menuLeaveSelected();
-        void menuForgetSelected();
         void refreshTitle();
 
     private:
-        QListView* view;
+        QTreeView* view;
         RoomListModel* model;
-        QSortFilterProxyModel* proxyModel;
-        QMenu* contextMenu;
+//        QSortFilterProxyModel* proxyModel;
+        QMenu* roomContextMenu;
+        QMenu* groupContextMenu;
         QAction* markAsReadAction;
         QAction* addTagsAction;
         QAction* joinAction;
         QAction* leaveAction;
         QAction* forgetAction;
+        QAction* deleteTagAction;
+        QVariant selectedGroupCache;
         QuaternionRoom* selectedRoomCache;
 
+        QVariant getSelectedGroup() const;
         QuaternionRoom* getSelectedRoom() const;
 };
